/**
 * Created by jmoutard on 19/07/16.
 */

'use strict';

angular.module('adf.electre')
    .service('storeService', [
        "$q", "serverStorageService", "localStorageService", "loggedIn",
        function ($q, serverStorageService, localStorageService, loggedIn) {
            /**
             * @description Return all stored dashboard from the server.
             * @returns {Promise}
             */
            function getAll () {
                var deferred = $q.defer(), values = {};

                if (loggedIn)
                    serverStorageService.getAll().then(function (value) {
                        values.serverStorage = value;
                        localStorageService.getAll().then(function (value) {
                            values.localStorage = value;
                        }, function () {
                            /* DO NOTHING */
                        }).then(function () {
                            deferred.resolve(values);
                        });
                    }, function (error) {
                        deferred.reject(error);
                    });
                else
                    localStorageService.getAll().then(function (value) {
                        values.localStorage = value;
                        deferred.resolve(values);
                    }, function (error) {
                        deferred.reject(error);
                    });

                return deferred.promise;
            }

            /**
             * @description Return a specific stored dashboard from the server.
             * @param {string} id
             * @returns {Promise}
             */
            function get (id) {
                var deferred = $q.defer();

                if (id === "localDashboard")
                    localStorageService.get(id)
                        .then(function (value) {
                            deferred.resolve(value);
                        }, function (error) {
                            deferred.reject(error);
                        });
                else
                    serverStorageService.get(id)
                        .then(function (value) {
                            deferred.resolve(value);
                        }, function (error) {
                            deferred.reject(error);
                        });

                return deferred.promise;
            }

            /**
             * @description Add a dashboard onto the server.
             * @param {string} id
             * @param {object} data
             * @returns {Promise}
             */
            function add (id, data) {
                var deferred = $q.defer();
                if (loggedIn && id !== "localDashboard")
                    serverStorageService.add(id, data).then(function (value) {
                        deferred.resolve(value);
                    }, function (error) {
                        deferred.reject(error);
                    });
                else
                    localStorageService.add(id, data).then(function (value) {
                        deferred.resolve(value);
                    }, function (error) {
                        deferred.reject(error);
                    });

                return deferred.promise;
            }

            /**
             * @description Remove a dashboard onto the server.
             * @param {string} id
             * @returns {Promise}
             */
            function remove (id) {
                var deferred = $q.defer();

                if (id === "localDashboard")
                    localStorageService.remove(id).then(function (value) {
                        deferred.resolve(value);
                    }, function (error) {
                        deferred.reject(error);
                    });
                else
                    serverStorageService.remove(id).then(function (value) {
                        deferred.resolve(value);
                    }, function (error) {
                        deferred.reject(error);
                    });

                return deferred.promise;
            }

            return { getAll: getAll, get: get, add: add, remove: remove};
        }
    ]);